/*===========================================================================
 * Project         : hardware4linux.info
 * Module          : hwreport
 * File            : scan-printers.c
 * Author          : Frederic Lepied
 * Created On      : Tue Feb 21 20:34:12 2006
 *===========================================================================*/

static const char rcs_id[] = "$Id$";
static const char compile_id[] = "$Compile: " __FILE__ " " __DATE__ " " __TIME__ " $";

/*
 **********************************************************************
 *
 * Included files
 *
 **********************************************************************
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>

#include "ioctl.h"
#include <linux/lp.h>

#ifndef IOCNR_GET_DEVICE_ID
#define IOCNR_GET_DEVICE_ID           1
#endif

#define LPIOC_GET_DEVICE_ID(len)      _IOC(_IOC_READ, 'P', IOCNR_GET_DEVICE_ID, len)

/*
 **********************************************************************
 *
 * Constants
 *
 **********************************************************************
 */

/*
 **********************************************************************
 *
 * Macros
 *
 **********************************************************************
 */

/*
 **********************************************************************
 *
 * New types
 *
 **********************************************************************
 */

/*
 **********************************************************************
 *
 * External functions
 *
 **********************************************************************
 */

/*
 **********************************************************************
 *
 * External variables
 *
 **********************************************************************
 */

/*
 **********************************************************************
 *
 * Global variables
 *
 **********************************************************************
 */

int
main(int argc, char * argv[])
{
  int id;
  char buf[1024];
  int status;
  int fd;
  char name[15];
  int len = sizeof(buf);
  int length;

  for(id=0; id<20; id++) {
    sprintf(name, "/dev/usb/lp%d", id);
    
    fd = open(name, O_RDONLY);

    if (fd >= 0) {
      memset(buf, 0, len);
      status = ioctl(fd, LPIOC_GET_DEVICE_ID(len), buf);
      close(fd);
      
      if (status == 0) {
	/*
	 * Extract the length of the device ID string from the first two
	 * bytes.  The 1284 spec says the length is stored MSB first...
	 */
	length = (((unsigned)buf[0] & 255) << 8) + ((unsigned)buf[1] & 255);
	
	/*
	 * Check to see if the length is larger than our buffer; first
	 * assume that the vendor incorrectly implemented the 1284 spec,
	 * and then limit the length to the size of our buffer...
	 */
	if (length > (len - 2))
	  length = (((unsigned)buf[1] & 255) << 8) + ((unsigned)buf[0] & 255);
	
	if (length > (len - 2))
	  length = len - 2;
	
	/*
	 * nul-terminate the device id.
	 */
	buf[length + 2] = '\0';

	printf("%s: %s\n", name, buf + 2);
      }
    } else {
      if (errno != ENODEV && errno != ENOENT) {
	perror("open");
      }
    }
  }
  
  return(0);
}


/*
 * Local variables:
 * mode: c
 * End:
 *
 * scan-printers.c ends here
 */
